﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.Data;
using System.Timers;


namespace CenteringAndRotatingWPF
{
    /// <summary>
    /// Interaction logic for TestWindow.xaml
    /// </summary>
    public partial class TestWindow : Window
    {
       
        public TestWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {           
            //Sets the correct map unit and the extent of the map.
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-66.8532, 10.4947, -66.8511, 10.4932);
            wpfMap1.Background = new SolidColorBrush(Color.FromRgb(243, 234, 167));

            LoadBackgroundLayers();

            SetCanvasRotate(60);
            wpfMap1.Refresh();          
        }

        private void LoadBackgroundLayers()
        {
            GdiPlusRasterLayer rasterLayer = new GdiPlusRasterLayer(@"C:\projects\IV\Images\Data_good\jpg.jpg");
            ValidateRasterLayer(rasterLayer);
            LayerOverlay rasterOverlay = new LayerOverlay();
            rasterOverlay.TileType = TileType.SingleTile;          
            rasterOverlay.Layers.Add("rasterLayer", rasterLayer);
            wpfMap1.Overlays.Add("raserOverlay", rasterOverlay);

            SetFullExtent(wpfMap1);
                      
        }

        public void SetCanvasRotate( double angle)
        {
            double actualHeight = this.Height;
            double actualWidth = this.Width;

            FrameworkElement element = GetElement(wpfMap1, "EventCanvas");
            FrameworkElement allEventCanvas = GetElement(wpfMap1, "AllEventCanvas");
            if (element != null && allEventCanvas != null)
            {
                if (allEventCanvas.ActualHeight < actualHeight * 2)
                {
                    allEventCanvas.Height = actualHeight * 2;
                    allEventCanvas.Width = actualWidth * 2;
                    allEventCanvas.Margin = new Thickness(-actualWidth / 2, -actualHeight / 2, 0, 0);
                }
                element.RenderTransform = new RotateTransform(angle, wpfMap1.ActualWidth / 2, wpfMap1.ActualHeight / 2);
            }
        }

        private FrameworkElement GetElement(FrameworkElement element, string name)
        {
            int count = VisualTreeHelper.GetChildrenCount(element);
            for (int i = 0; i < count; i++)
            {
                FrameworkElement child = VisualTreeHelper.GetChild(element, i) as FrameworkElement;
                if (child != null)
                {
                    if (child.Name == name)
                    {
                        return child;
                    }
                    else
                    {
                        var temp = GetElement(child, name);
                        if (temp != null && temp.Name == name)
                        {
                            return temp;
                        }
                    }
                }
            }
            return null;
        }

        private bool ValidateRasterLayer(RasterLayer rasterLayer)
        {
            bool isValid = true;

            try
            {
                rasterLayer.Open();
            }
            catch (Exception ex)
            {
                isValid = false;
            }
            finally
            {
                if (rasterLayer.IsOpen)
                {
                    rasterLayer.Close();
                }
            }

            return isValid;
        }
        /// <summary>
        /// Set the Extent of the map, so that all the layers are centered in the canvas view
        /// </summary>
        /// <param name="map"></param>
        public static void SetFullExtent(WpfMap map)
        {
            RectangleShape currentBoundingBox = GetFullExtent(map, (float)map.ActualWidth, (float)map.ActualHeight);
            if (currentBoundingBox != null)
            {
                map.CurrentExtent = currentBoundingBox;
                map.Refresh();
            }
        }

        /// <summary>
        /// Create RectangleShape that represent the minimal extent that will contain all the layers in the may
        /// </summary>
        /// <param name="map"></param>
        /// <param name="screenWidth">width of the screen (pixel)</param>
        /// <param name="screenHeight">height of the screen (pixel)</param>
        /// <returns>minimal rectangle area</returns>
        private static RectangleShape GetFullExtent(WpfMap map, float screenWidth, float screenHeight)
        {
            RectangleShape boundingBox = null;

            foreach (Overlay overlay in map.Overlays)
            {
                if (overlay is LayerOverlay)
                {
                    foreach (Layer layer in (overlay as LayerOverlay).Layers)
                    {
                        //ignore shape layers that do not contain any shapes
                        if (layer is InMemoryFeatureLayer && (layer as InMemoryFeatureLayer).InternalFeatures.Count <= 0)
                            continue;

                        if (layer.HasBoundingBox)
                        {
                            if (!layer.IsOpen) { layer.Open(); }
                            if (boundingBox == null)
                            {
                                boundingBox = layer.GetBoundingBox();
                            }
                            else
                            {
                                boundingBox.ExpandToInclude(layer.GetBoundingBox());
                            }
                        }
                    }
                }
            }

            if (boundingBox != null)
            {
                boundingBox = MapEngine.GetDrawingExtent(boundingBox, screenWidth, screenHeight);
            }

            return boundingBox;
        }
       
        private void wpfMap1_MouseMove(object sender, MouseEventArgs e)
        {
            //Gets the PointShape in world coordinates from screen coordinates.
            Point point = e.MouseDevice.GetPosition(null);

            ScreenPointF screenPointF = new ScreenPointF((float)point.X, (float)point.Y);
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(wpfMap1.CurrentExtent, screenPointF, (float)wpfMap1.Width, (float)wpfMap1.Height);

            //PointShape unrotatedPointShape = (PointShape)rotateProjection.ConvertToInternalProjection(pointShape);

           // textBox1.Text = "X: " + DecimalDegreesHelper.GetDegreesMinutesSecondsStringFromDecimalDegree(unrotatedPointShape.X) +
           //               "  Y: " + DecimalDegreesHelper.GetDegreesMinutesSecondsStringFromDecimalDegree(unrotatedPointShape.Y);

        }
    }
}
